@@ -27,24 +27,17 @@ app.initForm("<?cs var:html_escape(BBS_COOKIE_PATH) ?>");
 <style type="text/css">
 /*
 <![CDATA[
-<!--nobanner-->
 */
-a { color: <?cs var:html_escape(BBS_LINK_COLOR) ?>; }
-a:active { color: <?cs var:html_escape(BBS_ALINK_COLOR) ?>; }
-a:visited { color: <?cs var:html_escape(BBS_VLINK_COLOR) ?>; }
 fieldset { border: 0px; }
 img { border: 0px; }
 .center { text-align:center !important;  }
 .clear { clear: both !important; }
 .icon-align { vertical-align: top; }
 .image { border: none; float: left; }
-.nomail { color: <?cs var:html_escape(BBS_NAME_COLOR) ?> }
 .right { text-align: right !important; }
-.thread_info {}
 .res_body { margin-left: 40px; }
 /* ]]> */
 </style>
-<!--meta.txto͕-->
 <?cs var:META ?>
 </head>
 <body>
@@ -61,22 +54,15 @@ img { border: 0px; }
 <?cs /if ?>
 </div>
 
-<!--[J[\-->
 <div class="day" id="info">
  <h2><span class="title"><?cs var:html_escape(BBS_SUBTITLE) ?></span></h2>
  <div class="body">
   <div class="section">
    <?cs var:HEAD ?>
   </div>
-  <!--
-  <span class="center">
-   <a href="../index.html" id="backTo">fꗗ</a>
-  </span>
-  -->
  </div>
 </div>
 
-<!--L\-->
 <?cs if:Banner ?>
 <hr class="sep" />
 <div class="day" id="mainBanner">
@@ -91,7 +77,6 @@ img { border: 0px; }
 
 <hr class="sep" />
 
-<!--tH[\-->
 <div class="day" id="thread_builder_form">
  <h2><span class="title">XbhVK쐬</span></h2>
  <div class="form">
@@ -103,19 +88,30 @@ img { border: 0px; }
    <div class="textarea">
     ^CgF<input type="text" name="subject" size="25" />
    </div>
-   <div class="field name">
-    OF<input type="text" name="FROM" size="19" />
-   </div>
-   <div class="field mail">
-    E-mailF<input type="text" name="mail" size="19" />
+   <div>
+    <div class="field name">
+     OF<input type="text" name="FROM" size="19" />
+    </div>
+    <div class="field mail">
+     E-mailF<input type="text" name="mail" size="19" />
+    </div>
    </div>
    <?cs if:UploadableFromThread ?>
+   <div>
     <div class="field name">
-     <br />摜F<input type="file" name="file" />
+     摜F<input type="file" name="file" />
     </div>
     <div class="field mail">
      폜pXF<input type="password" name="pass" />
     </div>
+   </div>
+   <?cs /if ?>
+   <?cs if:Icon ?>
+   <div class="textarea">
+    ACRF<select name="icon"><option value="" selected="selected">Ȃ</option>
+    <?cs each:item = Icons ?><option value="<?cs var:item.id ?>"><?cs var:item.name ?></option>
+    <?cs /each ?></select>
+   </div>
    <?cs /if ?>
    <div class="textarea">
     RgF<textarea rows="5" cols="64" name="MESSAGE"></textarea>
