@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: GoogleVideo.pm 92 2007-01-16 13:47:05Z hikarin $
+# $Id: GoogleVideo.pm 643 2007-05-16 14:15:37Z hikarin $
 #
 
 package Img0ch::Plugin::Rewrite::StreamService::GoogleVideo;
@@ -25,14 +25,17 @@ sub new {
     my ( $iClass, $iBBS ) = @_;
     my $iConfig = $iBBS->get_kernel()->get_config();
 
-    bless [ ( $iConfig->get_int('googlevideo.width') || 425 ),
-            ( $iConfig->get_int('googlevideo.height') || 350 ),
+    bless [
+        ( $iConfig->get_int('googlevideo.width')  || 425 ),
+        ( $iConfig->get_int('googlevideo.height') || 350 ),
+        ( $iConfig->get('EnableXHTMLBreak') ? ' />' : '>' ),
     ], $iClass;
 }
 
 sub from_bbs {
     my ( $iObject, $key, $count, $resno, $text ) = @_;
-    my ( $width, $height ) = ( $iObject->[0], $iObject->[1] );
+    my ( $width, $height, $html_close_tag )
+        = ( $iObject->[0], $iObject->[1], $iObject->[2] );
 
     $$text =~ s{
         (?:\<a[^>]+>)?
@@ -40,11 +43,10 @@ sub from_bbs {
         (?:\</a>)?
         }
         {<object width="${width}" height="${height}">
-<param name="movie"
-  value="http://video.google.com/googleplayer.swf?docId=$1&hl=en" />
+<param name="movie" value="http://video.google.com/googleplayer.swf?docId=$1&hl=en"${html_close_tag}
 <embed src="http://video.google.com/googleplayer.swf?docId=$1&hl=en"
-  type="application/x-shockwave-flash" width="${width}" height="${height}" />
-</object><br />}gxms;
+  type="application/x-shockwave-flash" width="${width}" height="${height}"${html_close_tag}
+</object><br${html_close_tag}}gxms;
     return;
 }
 
