@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Upload.pm 198 2007-02-08 13:39:47Z hikarin $
+# $Id: Upload.pm 718 2007-05-29 13:19:56Z hikarin $
 #
 
 package Img0ch::Upload;
@@ -24,26 +24,32 @@ use Digest::MD5 qw();
 
 sub new {
     my ( $iClass, $iBBS, $key ) = @_;
-    my $bbs     = $iBBS->get_name();
-    my $iKernel = $iBBS->get_kernel();
-
+    my $iKernel   = $iBBS->get_kernel();
     my $repos_tag = $iBBS->get_repos_path('tag');
     my $iReposTag = $iKernel->get_repos($repos_tag);
+    my $bbs       = $iBBS->get_name();
 
-    my ( undef, undef, undef, $day, $month, $year ) = localtime($key);
-    $year  += 1900;
-    $month += 1;
-    my $repos_upload = $iBBS->get_repos_path(
-        $year, $month, $day, $key, 'upload' );
-    my $repos_upload_dir = $repos_upload;
-    $repos_upload_dir =~ s{/upload.\w+\z}{}xms;
-    if ( !-d $repos_upload_dir ) {
-        require File::Path;
-        File::Path::mkpath($repos_upload_dir)
-            or $iKernel->throw_io_exception($repos_upload_dir);
+    my $repos_upload = '';
+    if ($key) {
+        my ( undef, undef, undef, $day, $month, $year ) = localtime($key);
+        $year  += 1900;
+        $month += 1;
+        $repos_upload
+            = $iBBS->get_repos_path( $year, $month, $day, $key, 'upload' );
+        my $repos_upload_dir = $repos_upload;
+        $repos_upload_dir =~ s{/upload.\w+\z}{}xms;
+        if ( !-d $repos_upload_dir ) {
+            require File::Path;
+            File::Path::mkpath($repos_upload_dir)
+                or $iKernel->throw_io_exception($repos_upload_dir);
+        }
     }
 
-    my $iReposUpload = $iKernel->get_repos($repos_upload);
+    my ( $iReposUpload, $count ) = ( '', 0 );
+    if ($repos_upload) {
+        $iReposUpload = $iKernel->get_repos($repos_upload);
+        $count        = $iReposUpload->get_int("I:U:I.${bbs}.${key}._");
+    }
     if ( $iKernel->get_config()->get('compatible.upload') ) {
         require Img0ch::Compat::Upload;
         Img0ch::Compat::Upload::load( $iReposUpload, $iBBS, $key );
@@ -51,30 +57,32 @@ sub new {
 
     bless {
         _i0_bbs  => $iBBS,
-        _count   => $iReposUpload->get_int("I:U:I.${bbs}.${key}._"),
+        _count   => $count,
         _dir     => $bbs,
         __dlpass => '',
         _kernel  => $iKernel,
         _key     => $key,
         _path    => $iBBS->path( 'img', $key ),
+        __prefix => "I:U:I.${bbs}.${key}",
         __rmpass => '',
         _rs      => $iReposUpload,
         _tag     => $iReposTag,
         __upload => [],
-        __url    => $iBBS->url( 'img', $key ),
+        _url     => $iBBS->url( 'img', $key ),
         __path   => '',
     }, $iClass;
 }
 
 sub load {
     my ($iUpload) = @_;
-    $iUpload->{_rs}->load();
+    my $iReposUpload = $iUpload->{_rs};
+    $iReposUpload and $iReposUpload->load();
     $iUpload->{_tag}->load();
     return 1;
 }
 
 sub save {
-    my ($iUpload) = @_;
+    my ($iUpload)    = @_;
     my $iReposUpload = $iUpload->{_rs};
     my $line         = $iUpload->{__upload};
     if ( !@$line ) {
@@ -102,13 +110,15 @@ sub save {
     }
 
     my ( $width, $height ) = $iUpload->get_dimensions($src);
-    my $prefix       = "I:U:I.${bbs}.${key}.${resno}";
+    my $prefix = "I:U:I.${bbs}.${key}.${resno}";
     $iReposUpload->set( "${prefix}.extension", $ext );
     $iReposUpload->set( "${prefix}.width",     $width );
     $iReposUpload->set( "${prefix}.height",    $height );
     $iReposUpload->set( "${prefix}.hash",      $iUpload->get_digest($src) );
-    $file_not_uploaded and $iReposUpload->set(
-        "I:U:I.${bbs}.${key}._", ++$iUpload->{_count} );
+    $file_not_uploaded
+        and
+        $iReposUpload->set( "I:U:I.${bbs}.${key}._", ++$iUpload->{_count} );
+
     if ( my $dlkey = $iUpload->{__dlpass} ) {
         chmod 0600, $path;
         $iReposUpload->set( "${prefix}.download.pass",
@@ -121,33 +131,72 @@ sub save {
         $iUpload->{__rmpass} = '';
     }
 
-    @{$iUpload->{__upload}} = ();
+    @{ $iUpload->{__upload} } = ();
     $iUpload->{__path} = $path;
     $iReposUpload->save();
     return 1;
 }
 
+sub update_csv {
+    my ( $iUpload, $max ) = @_;
+    $iUpload->count() or return 0;
+    $max = Img0ch::Kernel::intval($max);
+
+    my $iKernel = $iUpload->{_kernel};
+    my $path    = $iUpload->path() . '/upload.csv';
+    my $temp    = $path . '.tmp';
+    my $LOCK_EX = 2;
+
+    eval {
+        require Fcntl;
+        $LOCK_EX = Fcntl::LOCK_EX();
+    };
+    eval {
+        local ( $!, *FH );
+        open *FH, ">${temp}"    ## no critic
+            or $iKernel->throw_io_exception($path);
+        flock *FH, $LOCK_EX;
+        my $count = 0;
+        for ( my $i = 0; $i <= $max; $i++ ) {
+            my $info = $iUpload->get($i);
+            if ( my $ext = $info->[0] ) {
+                my ( $w, $h ) = ( 0, 0 );
+                $iUpload->get_thumbnail_size( $i, \$w, \$h );
+                print {*FH} $i, ',', $ext, ',', $info->[1], ',', $info->[2],
+                    ',', $info->[3], ',', $w, ',', $h, "\n"
+                    or $iKernel->throw_io_exception($path);
+            }
+        }
+        close *FH or $iKernel->throw_io_exception($path);
+        rename $temp, $path or $iKernel->throw_io_exception($path);
+    };
+    if ( my $exception = $@ ) {
+        unlink $temp or $iKernel->throw_io_exception($temp);
+        die $exception, "\n";
+    }
+
+    return 1;
+}
+
 sub get {
     my ( $iUpload, $resno ) = @_;
-    my ( $bbs, $key ) = ( $iUpload->{_dir}, $iUpload->{_key} );
-    my $prefix       = "I:U:I.${bbs}.${key}.${resno}";
+    my $prefix       = join '.', $iUpload->{__prefix}, $resno;
     my $iReposUpload = $iUpload->{_rs};
 
-    if ( my $ext = $iReposUpload->get("${prefix}.extension") ) {
+    if ( my $hash = $iReposUpload->get("${prefix}.hash") ) {
+        my $ext    = $iReposUpload->get("${prefix}.extension");
         my $width  = $iReposUpload->get_int("${prefix}.width");
         my $height = $iReposUpload->get_int("${prefix}.height");
-        my $hash   = $iReposUpload->get("${prefix}.hash");
         return [ $ext, $width, $height, $hash ];
     }
     else {
-        return [ '', 0, 0, '0000000000000000000000' ];
+        return [ '', 0, 0, '' ];
     }
 }
 
 sub get_thumbnail_size {
     my ( $iUpload, $resno, $ref_width, $ref_height ) = @_;
-    my ( $bbs, $key ) = ( $iUpload->{_dir}, $iUpload->{_key} );
-    my $prefix       = "I:U:I.${bbs}.${key}.${resno}";
+    my $prefix       = join '.', $iUpload->{__prefix}, $resno;
     my $iReposUpload = $iUpload->{_rs};
 
     if ( my $width = $iReposUpload->get_int("${prefix}.width.thumbnail") ) {
@@ -161,7 +210,7 @@ sub get_thumbnail_size {
 
 sub set {
     my ( $iUpload, $src, $resno, $ext ) = @_;
-    @{$iUpload->{__upload}} = ( $src, $resno, $ext );
+    @{ $iUpload->{__upload} } = ( $src, $resno, $ext );
     return;
 }
 
@@ -175,8 +224,8 @@ sub set_key {
     my ( undef, undef, undef, $day, $month, $year ) = localtime($key);
     $year  += 1900;
     $month += 1;
-    my $repos_upload = $iBBS->get_repos_path(
-        $year, $month, $day, $key, 'upload' );
+    my $repos_upload
+        = $iBBS->get_repos_path( $year, $month, $day, $key, 'upload' );
     my $repos_upload_dir = $repos_upload;
     $repos_upload_dir =~ s{/upload.\w+\z}{}xms;
     if ( !-d $repos_upload_dir ) {
@@ -185,11 +234,13 @@ sub set_key {
             or $iKernel->throw_io_exception($repos_upload_dir);
     }
 
-    my $iReposUpload = $iKernel->get_repos($repos_upload);
-    $iUpload->{_count} = $iReposUpload->get_int("I:U:I.${bbs}.${key}._");
-    $iUpload->{_path}  = $iBBS->path( 'img', $key );
-    $iUpload->{_rs}    = $iReposUpload;
-    $iUpload->{_url}   = $iBBS->url( 'img', $key );
+    my $prefix           = "I:U:I.${bbs}.${key}";
+    my $iReposUpload     = $iKernel->get_repos($repos_upload);
+    $iUpload->{_count}   = $iReposUpload->get_int("${prefix}._");
+    $iUpload->{_path}    = $iBBS->path( 'img', $key );
+    $iUpload->{_rs}      = $iReposUpload;
+    $iUpload->{_url}     = $iBBS->url( 'img', $key );
+    $iUpload->{__prefix} = $prefix;
     return;
 }
 
@@ -197,13 +248,18 @@ sub set_download_password { $_[0]->{__dlpass} = $_[1]; return; }
 
 sub set_remove_password { $_[0]->{__rmpass} = $_[1]; return; }
 
+sub have_download_password {
+    my ( $iUpload, $resno ) = @_;
+    my $iReposUpload = $iUpload->{_rs};
+    return $iReposUpload->get(
+        join '.', $iUpload->{__prefix}, $resno, 'download.pass') ? 1 : 0;
+}
+
 sub set_thumbnail_size {
     my ( $iUpload, $resno, $width, $height ) = @_;
-    my ( $bbs, $key ) = ( $iUpload->{_dir}, $iUpload->{_key} );
-    my $prefix       = "I:U:I.${bbs}.${key}.${resno}";
     my $iReposUpload = $iUpload->{_rs};
-
-    $iReposUpload->set( "${prefix}.width.thumbnail", $width );
+    my $prefix       = join '.', $iUpload->{__prefix}, $resno;
+    $iReposUpload->set( "${prefix}.width.thumbnail",  $width );
     $iReposUpload->set( "${prefix}.height.thumbnail", $height );
     return;
 }
@@ -218,7 +274,7 @@ sub path {
 
 sub url {
     my ( $iUpload, $resno, $ext ) = @_;
-    my $path = $iUpload->{__url};
+    my $path = $iUpload->{_url};
     ( $resno and $ext ) ? "${path}/${resno}.${ext}" : $path;
 }
 
@@ -230,8 +286,7 @@ sub can_remove { shift->_can( 'remove', @_ ) }
 
 sub _can {
     my ( $iUpload, $type, $resno, $pass ) = @_;
-    my ( $bbs, $key ) = ( $iUpload->{_dir}, $iUpload->{_key} );
-    my $key_by_type  = "I:U:I.${bbs}.${key}.${resno}.${type}.pass";
+    my $key_by_type  = join '.', $iUpload->{__prefix}, $resno, $type, 'pass';
     my $iReposUpload = $iUpload->{_rs};
 
     if ( my $pass_md5 = $iReposUpload->get($key_by_type) ) {
@@ -252,7 +307,7 @@ sub get_digest {
     binmode *FH;
     my $md5 = Digest::MD5->new();
     while ( read *FH, my $buffer, 32768 ) {
-        $md5->add( $buffer );
+        $md5->add($buffer);
     }
     my $digest = $md5->b64digest();
     close *FH or $iKernel->throw_io_exception($path);
@@ -263,7 +318,8 @@ sub get_dimensions {
     my ( $iClass, $path ) = @_;
     my ( $width, $height ) = ( 0, 0 );
 
-    if ( $path =~ /\.jpg\z/xms and eval 'use Image::Epeg qw(); 1' ) {
+    if ( $path =~ /\.jpg\z/xms and ( defined $INC{'Image/Epeg.pm'}
+        or eval 'use Image::Epeg qw(); 1' ) ) {
         my $epeg = Image::Epeg->new($path);
         if ($epeg) {
             $width  = $epeg->get_width();
@@ -279,18 +335,22 @@ sub get_dimensions {
     #            $height = $imlib->get_height;
     #        }
     #    }
-    elsif ( eval 'use GD qw(); 1' ) {
-        ( $width, $height ) = sub {
+    elsif ( defined $INC{'GD.pm'} or eval 'use GD qw(); 1' ) {
+        my $gd = sub {
             return GD::Image->newFromJpeg($path) if $path =~ /\.jpg\z/xms;
             return GD::Image->newFromPng($path)  if $path =~ /\.png\z/xms;
             if ( $path =~ /\.gif\z/xms and GD->VERSION() >= 2.15 ) {
                 return GD::Image->newFromGif($path) || GD::Image->new( 0, 0 );
             }
-            return GD::Image->new( 0, 0 );
+            return;
             }
-            ->()->getBounds();
+            ->();
+        if ($gd) {
+            ( $width, $height ) = $gd->getBounds();
+        }
     }
-    elsif ( eval 'use Image::Magick qw(); 1' ) {
+    elsif ( defined $INC{'Image/Magick.pm'}
+        or eval 'use Image::Magick qw(); 1' ) {
         my $image = Image::Magick->new;
         ( $width, $height ) = $image->Ping($path);
     }
@@ -322,7 +382,7 @@ sub get_tags {
             [
             $iReposTag->get("${prefix}.bbs"),
             $iReposTag->get("${prefix}.key"),
-            $iReposTag->get("${prefix}.resno"),
+            $iReposTag->get_int("${prefix}.resno"),
             $tag,
             ];
     }
@@ -360,14 +420,14 @@ sub get_tag_count {
 }
 
 sub set_tags {
-    my ( $iUpload, $comment, $max ) = @_;
+    my ( $iUpload, $comment, $max, $resno, $no_commit ) = @_;
     $$comment =~ /<br>/xms or return 0;
     $max = Img0ch::Kernel::intval($max) || 3;
+    $resno ||= $iUpload->{__upload}->[1];
 
     my $iReposTag = $iUpload->{_tag};
     my $bbs       = $iUpload->{_dir};
     my $key       = $iUpload->{_key};
-    my $resno     = $iUpload->{__upload}->[1];
     my $line      = [ split '<br>', $$comment ]->[1] || '';
     my %seen      = ();
     my $i         = 0;
@@ -388,7 +448,7 @@ sub set_tags {
         $iReposTag->set( "${prefix}.resno",    $resno );
     }
 
-    $iReposTag->save();
+    !$no_commit and $iReposTag->save();
     return $i;
 }
 
@@ -406,9 +466,9 @@ sub scale {
 }
 
 sub traverse {
-    my ( $iUpload ) = @_;
-    my $iKernel = $iUpload->{_kernel};
-    my $path    = $iUpload->path();
+    my ($iUpload) = @_;
+    my $iKernel   = $iUpload->{_kernel};
+    my $path      = $iUpload->path();
     -d $path or return 0;
 
     local ( $!, *DH );
@@ -423,23 +483,24 @@ sub traverse {
 
 sub remove {
     my ( $iUpload, $resno_or_path ) = @_;
-    my $iReposUpload  = $iUpload->{_rs};
-    my ( $bbs, $key ) = ( $iUpload->{_dir}, $iUpload->{_key} );
+    my $iReposUpload = $iUpload->{_rs};
 
     my $resno;
-    if (Img0ch::Kernel::intval($resno_or_path)) {
+    if ( Img0ch::Kernel::intval($resno_or_path) ) {
         $resno = $resno_or_path;
     }
     else {
         -r $resno_or_path or return 0;
         ($resno) = $resno_or_path =~ m{/t?(\d+)\.[\w\.]+\z}xms;
     }
-    my $prefix        = "I:U:I.${bbs}.${key}.${resno}";
-    my $extension     = $iReposUpload->get("${prefix}.extension");
+    my $prefix_top = $iUpload->{__prefix};
+    my $prefix     = join '.', $prefix_top, $resno;
+    my $extension  = $iReposUpload->get("${prefix}.extension");
 
     my $uploaded_file_path = $iUpload->path( $resno, $extension );
     if ( ( -f $uploaded_file_path or -l $uploaded_file_path )
-        and -w $uploaded_file_path ) {
+        and -w $uploaded_file_path )
+    {
         my $iKernel = $iUpload->{_kernel};
         unlink $uploaded_file_path
             or $iKernel->throw_io_exception($uploaded_file_path);
@@ -457,8 +518,8 @@ sub remove {
     $iReposUpload->remove("${prefix}.width.thumbnail");
     $iReposUpload->remove("${prefix}.height.thumbnail");
     $iReposUpload->remove("${prefix}.download.pass");
-    $iReposUpload->set( "${prefix}.hash", $iUpload->removed_mark() );
-    $iReposUpload->set( "I:U:I.${bbs}.${key}._", --$iUpload->{_count} );
+    $iReposUpload->set( "${prefix}.hash",        $iUpload->removed_mark() );
+    $iReposUpload->set( "${prefix_top}._", --$iUpload->{_count} );
     return 1;
 }
 
