@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: App.pm 400 2007-03-21 13:59:23Z hikarin $
+# $Id: App.pm 740 2007-06-01 13:40:42Z hikarin $
 #
 
 package Zeromin::App;
@@ -43,7 +43,6 @@ sub run {
     my $iConfig  = $zApp->kernel()->get_config();
     my $iRequest = $zApp->{__request};
     my $content;
-    $iRequest->init($iConfig);
 
     $zApp->proxy( \$content, $iRequest );
     if ( $iConfig->get('ZerominUseGzip') ) {
@@ -87,7 +86,7 @@ sub proxy {
     eval {
         my $mtd = $zApp->load( $Qclass, $method )
             or die "${Qclass}::${method}() not found\n";
-        $iRequest->init();
+        $iRequest->init($zApp->kernel()->get_config());
         if ( my $bbs = $iRequest->bbs() ) {
             require Img0ch::BBS;
             $zApp->{__bbs}
@@ -216,7 +215,7 @@ sub unijp {
 
 sub _html {
     my ( $zApp, $content, $iRequest, $parameters, $t_dir, $t_file ) = @_;
-    $parameters ||= {};
+    $parameters             ||= {};
     $zApp->{__content_type} ||= 'text/html';
 
     require Img0ch::Template;
@@ -239,7 +238,7 @@ sub _html {
         $zApp->{__kernel},
         {   dir     => $dir,
             file    => $file,
-            version => $iRequest->credit(),
+            request => $iRequest,
         }
     );
     my $iConfig = $zApp->kernel()->get_config();
