@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Plugin.pm 164 2007-02-02 14:52:06Z hikarin $
+# $Id: Plugin.pm 653 2007-05-19 12:51:24Z hikarin $
 #
 
 package Zeromin::Plugin;
@@ -22,7 +22,6 @@ package Zeromin::Plugin;
 use strict;
 use base qw(Img0ch::Plugin);
 use Module::Pluggable::Object qw();
-use Storable qw();
 
 sub new {
     my ( $zClass, $iObject, $key ) = @_;
@@ -33,7 +32,7 @@ sub new {
         $zPlugin = $zClass->SUPER::new( $iObject, $key );
         $zPlugin->{__kernel} = $iObject->get_kernel();
     }
-    elsif ( $class eq 'Img0ch::Maple' ) {
+    elsif ( $class eq 'Img0ch::Kernel' or $class eq 'Img0ch::Maple' ) {
         require Img0ch::BBS;
         $zPlugin
             = $zClass->SUPER::new( Img0ch::BBS->new( $iObject, { id => 0 } ),
@@ -42,7 +41,7 @@ sub new {
     }
     else {
         Img0ch::Kernel->throw_exception(
-            'Img0ch::BBS or Img0ch::Maple not given');
+            'Img0ch::BBS or Img0ch::Maple(Img0ch::Kernel) not given');
     }
 
     my $type2num = $zPlugin->{_type2num};
@@ -128,8 +127,8 @@ sub load {
         ? ( split ',', $enabled )
         : ($enabled)
     );
-    my @enabled_plugins = grep {
-        Img0ch::Kernel::intval($_) != 0 } keys %$duplicate_check_hash;
+    my @enabled_plugins = grep { Img0ch::Kernel::intval($_) != 0 }
+        keys %$duplicate_check_hash;
 
     %$duplicate_check_hash = ();
     map { $duplicate_check_hash->{$_} = 1 } (
@@ -137,8 +136,8 @@ sub load {
         ? ( split ',', $disabled )
         : ($disabled)
     );
-    my @disabled_plugins = grep {
-        Img0ch::Kernel::intval($_) != 0 } keys %$duplicate_check_hash;
+    my @disabled_plugins = grep { Img0ch::Kernel::intval($_) != 0 }
+        keys %$duplicate_check_hash;
 
     my $closure = sub {
         my ( $found, $enabled, $plugins ) = @_;
@@ -182,9 +181,10 @@ sub save {
                 $iRepos->set( "I:P:E.${i}.class",  $class );
                 $iRepos->set( "I:P:E.${i}.method", $method );
                 $iRepos->set( "I:P:E.${i}.type",   $type );
+
                 # $value->[0] = $i; $i++;
-                $value->[0] = $i++;
-                $value->[3] = 0;
+                $value->[0]             = $i++;
+                $value->[3]             = 0;
                 $class_table->{$method} = $value;
             }
             else {
@@ -193,9 +193,10 @@ sub save {
         }
         %{ $loaded->{$class} } = %{$class_table};
     }
+
     # $zPlugin->{__lastid} = $i; $i++
     $zPlugin->{__lastid} = $i++;
-    $iRepos->set('I:P:E._', $i);
+    $iRepos->set( 'I:P:E._', $i );
     %{ $zPlugin->{__load} } = %{$loaded};
 
     while ( my ( $type_name, $plugin_ids ) = each %{$typemap} ) {
@@ -318,10 +319,10 @@ sub copy {
 
 sub all {
     my ( $zPlugin, $zPluginMergeFrom ) = @_;
-    my $load      = $zPlugin->{__load};
-    my $mf_load   = $zPluginMergeFrom->{__load};
-    my $num2type  = $zPlugin->{__num2type};
-    my $ret       = {};
+    my $load     = $zPlugin->{__load};
+    my $mf_load  = $zPluginMergeFrom->{__load};
+    my $num2type = $zPlugin->{__num2type};
+    my $ret      = {};
 
     while ( my ( $class, $entry1 ) = each %{$load} ) {
         $class or next;
@@ -341,11 +342,39 @@ sub all {
     $ret;
 }
 
+sub filter_with_page {
+    my ( $zPlugin, $zPluginMergeFrom, $query, $item_per_page, $offset ) = @_;
+    my $classes = {};
+    my $content = [];
+    my $plugins = $zPlugin->all($zPluginMergeFrom);
+
+    while ( my ( $id, $entry ) = each %$plugins ) {
+        my $class = $entry->{class};
+        $classes->{$class} = 1;
+        ( $query and $class ne $query ) and next;
+        push @$content,
+            {
+            id      => $id,
+            class   => $class,
+            method  => $entry->{method},
+            type    => $entry->{type},
+            enabled => $entry->{enabled}
+            };
+    }
+
+    my $class = [];
+    map { push @$class, { name => $_ } } sort keys %$classes;
+
+    require Data::Page;
+    my $page = Data::Page->new( scalar @$content, $item_per_page, $offset );
+    return ( [ $page->splice($content) ], $page, $class );
+}
+
 sub retrive {
     my ( $zPlugin, $class, $bbs, $key ) = @_;
     $class ||= '';
-    my $id =
-          $class =~ /\A\d+\z/xms
+    my $id
+        = $class =~ /\A\d+\z/xms
         ? $class
         : $zPlugin->get_id( $class, $bbs );
     $zPlugin->SUPER::retrive( $id, $bbs, $key );
@@ -354,8 +383,8 @@ sub retrive {
 sub store {
     my ( $zPlugin, $class, $bbs, $key, $data ) = @_;
     $class ||= '';
-    my $id =
-          $class =~ /\A\d+\z/xms
+    my $id
+        = $class =~ /\A\d+\z/xms
         ? $class
         : $zPlugin->get_id( $class, $bbs );
     $zPlugin->SUPER::store( $id, $bbs, $key, $data );
@@ -368,8 +397,8 @@ sub disable { _switch( @_, 0 ) }
 
 sub is_enabled {
     my ( $zPlugin, $class ) = @_;
-    my ( $cls, $mtd ) =
-          $class =~ /\A(\d+)\z/xms
+    my ( $cls, $mtd )
+        = $class =~ /\A(\d+)\z/xms
         ? $zPlugin->get_plugin($1)
         : _split($class);
     $zPlugin->{__load}->{$cls}->{$mtd}->[2] ? 1 : 0;
@@ -423,6 +452,7 @@ sub install {
             $merge->{$method}->[1] = $num;
         }
         else {
+
             # pid, type, default is disabled, installing
             $merge->{$method} = [ 0, $num, 0, 1 ];
             $zPlugin->{__installed}++;
@@ -455,178 +485,11 @@ sub uninstall {
 
 sub set_form {
     my ( $zPlugin, $id, $bbs, $key ) = @_;
-    require HTML::Element;
-    $zPlugin->{__plugin_config_html} ||= HTML::Element->new('div');
-    $zPlugin->{__plugin_config_id}    = $id;
-    $zPlugin->{__plugin_config_bbs}   = $bbs || $zPlugin->{_bbs} || 0;
-    $zPlugin->{__plugin_config_key}   = $key || $zPlugin->{_key} || 0;
-    $zPlugin->{__plugin_config_fkeys} = [];
-    $zPlugin->{__plugin_config_js}    = '';
-    return;
-}
-
-sub add_input_text_form {
-    my ( $zPlugin, $name, $attrs ) = @_;
-    $zPlugin->_add_input_form( $name, $attrs, 'text' );
-}
-
-sub add_input_password_form {
-    my ( $zPlugin, $name, $attrs ) = @_;
-    $zPlugin->_add_input_form( $name, $attrs, 'password' );
-}
-
-sub add_input_hidden_form {
-    my ( $zPlugin, $name, $attrs ) = @_;
-    $zPlugin->_add_input_form( $name, $attrs, 'hidden' );
-}
-
-sub _add_input_form {
-    my ( $zPlugin, $name, $attrs, $type ) = @_;
-    my $data = $zPlugin->retrive(
-        $zPlugin->{__plugin_config_id},
-        $zPlugin->{__plugin_config_bbs},
-        $zPlugin->{__plugin_config_key}, $name,
-    );
-
-    $attrs ||= {};
-    my $tag = HTML::Element->new('input');
-    while ( my ( $key, $value ) = each %{$attrs} ) {
-        $tag->attr( $key, $value );
-    }
-    $tag->attr( 'name',  $name );
-    $tag->attr( 'type',  $type );
-    $tag->attr( 'value', $data->{$name} || '' );
-    $zPlugin->{__plugin_config_html}->push_content($tag);
-    push @{ $zPlugin->{__plugin_config_fkeys} }, $name;
-    return;
-}
-
-sub add_textarea_form {
-    my ( $zPlugin, $name, $attrs ) = @_;
-    my $data = $zPlugin->retrive(
-        $zPlugin->{__plugin_config_id},
-        $zPlugin->{__plugin_config_bbs},
-        $zPlugin->{__plugin_config_key}, $name,
-    );
-
-    $attrs ||= {};
-    my $tag = HTML::Element->new('textarea');
-    while ( my ( $key, $value ) = each %{$attrs} ) {
-        $tag->attr( $key, $value );
-    }
-    $tag->attr( 'name', $name );
-    my $data_array = $data->{$name} || [];
-    $tag->push_content( join "\n", @{$data_array} );
-    $zPlugin->{__plugin_config_html}->push_content($tag);
-    push @{ $zPlugin->{__plugin_config_fkeys} }, $name;
-    return;
-}
-
-sub add_select_form {
-    my ( $zPlugin, $name, $values, $multiple, $attrs ) = @_;
-    my $data = $zPlugin->retrive(
-        $zPlugin->{__plugin_config_id},
-        $zPlugin->{__plugin_config_bbs},
-        $zPlugin->{__plugin_config_key}, $name,
-    );
-
-    $attrs  ||= {};
-    $values ||= [];
-    my $tag = HTML::Element->new('select');
-    while ( my ( $key, $value ) = each %{$attrs} ) {
-        $tag->attr( $key, $value );
-    }
-    $tag->attr( 'name', $name );
-    $multiple and $tag->attr( 'multiple', 'multiple' );
-    my $data_array = $data->{$name} || [];
-    for my $value ( @{$values} ) {
-        ( ref $value || '' ) eq 'ARRAY' or next;
-        my $opt        = HTML::Element->new('option');
-        my $value_attr = $value->[0];
-        $opt->attr( 'value', $value_attr );
-        if ( ( grep { $value_attr eq $_ } @{$data_array} ) > 0 ) {
-            $opt->attr( 'selected', 'selected' );
-        }
-        $opt->push_content( $value->[1] );
-        $tag->push_content($opt);
-    }
-    $zPlugin->{__plugin_config_html}->push_content($tag);
-    push @{ $zPlugin->{__plugin_config_fkeys} }, $name;
-    return;
-}
-
-sub add_text {
-    my ( $zPlugin, $text ) = @_;
-    $zPlugin->{__plugin_config_html}->push_content($text);
-    return;
-}
-
-sub set_javascript_code {
-    my ( $zPlugin, $jscode ) = @_;
-    $zPlugin->{__plugin_config_js} = $jscode;
-    return;
-}
-
-sub render_html {
-    my ($zPlugin) = @_;
-    my $he = $zPlugin->{__plugin_config_html};
-
-    for my $name ( @{ $zPlugin->{__plugin_config_fkeys} } ) {
-        my $hidden = HTML::Element->new('input');
-        $hidden->attr( 'name',  'config' );
-        $hidden->attr( 'type',  'hidden' );
-        $hidden->attr( 'value', $name );
-        $he->push_content($hidden);
-    }
-    $he->push_content(
-        HTML::Element->new(
-            'input',
-            'name'  => 'bbs',
-            'type'  => 'hidden',
-            'value' => $zPlugin->{__plugin_config_bbs},
-        ),
-        HTML::Element->new(
-            'input',
-            'name'  => 'key',
-            'type'  => 'hidden',
-            'value' => $zPlugin->{__plugin_config_key},
-        ),
-        HTML::Element->new(
-            'input',
-            'name'  => 'id',
-            'type'  => 'hidden',
-            'value' => $zPlugin->{__plugin_config_id},
-        ),
-    );
-    @{ $zPlugin->{__plugin_config_fkeys} } = ();
-
-    my $html = $he->as_HTML();
-    $he->delete_content();
-    return \$html;
-}
-
-sub render_javascript {
-    my ($zPlugin) = @_;
-    return \$zPlugin->{__plugin_config_js};
-}
-
-sub serialize {
-    my ( $zPlugin, $param ) = @_;
-    my $ret = {};
-    while ( my ( $key, $value ) = each %$param ) {
-        my $reftype = ref $value || '';
-        if ( $reftype eq 'ARRAY' and @$value <= 1 ) {
-            $ret->{$key} = join '', @$value;
-            next;
-        }
-        elsif ( $reftype eq '' and index( $value, "\n" ) >= 0 ) {
-            $value =~ s/\r\n/\n/gxms;
-            $ret->{$key} = [ split "\n", $value ];
-            next;
-        }
-        $ret->{$key} = $value;
-    }
-    return $ret;
+    $bbs ||= $zPlugin->{_bbs} || 0;
+    $key ||= $zPlugin->{_key} || 0;
+    require Zeromin::Plugin::Config;
+    my $zPConfig = Zeromin::Plugin::Config->new( $zPlugin, $id, $bbs, $key );
+    return $zPConfig;
 }
 
 sub remove_from_bbs {
@@ -658,11 +521,11 @@ sub _split { my @ret = ( $_[0] || '.::.' ) =~ /\A(.+)::(.+)\z/xms }
 
 sub _switch {
     my ( $zPlugin, $class, $bit ) = @_;
-    my ( $cls, $mtd ) =
-          $class =~ /\A(\d+)\z/xms
+    my ( $cls, $mtd )
+        = $class =~ /\A(\d+)\z/xms
         ? $zPlugin->get_plugin($1)
         : _split($class);
-    exists $zPlugin->{__load}->{$cls} or return 0;
+    exists $zPlugin->{__load}->{$cls}         or return 0;
     exists $zPlugin->{__load}->{$cls}->{$mtd} or return 0;
     $zPlugin->{__load}->{$cls}->{$mtd}->[2] = $bit;
     return 1;
