@@ -14,7 +14,7 @@
 # *  limitations under the License.
 # */
 #
-# $Id: Thread.pm 73 2007-01-06 12:16:49Z hikarin $
+# $Id: Thread.pm 647 2007-05-16 14:20:58Z hikarin $
 #
 
 package Zeromin::Thread;
@@ -26,8 +26,9 @@ sub get_bbs { Img0ch::BBS->new( $_[0]->{_kernel}, { id => $_[0]->{_bbs} } ) }
 
 sub set {
     my ( $zThread, $element, $pos ) = @_;
-    if ($pos) {
-        my $subject = $element->[5] || '';
+    if ( $pos = Img0ch::Kernel::intval($pos) ) {
+        my $subject = ( $pos == 1
+            ? ($element->[5] || $zThread->get_subject()) : '' );
         my $id = $element->[3] ? " $element->[3]" : '';
         my $line = "$element->[0]<>$element->[1]<>$element->[2]"
             . "$id<>$element->[4]<>${subject}\n";
@@ -104,7 +105,9 @@ sub remove {
         }
         require Zeromin::Subject;
         my $zSubject = Zeromin::Subject->new($iBBS);
+        $zSubject->load();
         $zSubject->remove($key);
+        $zSubject->include_special_threads();
         $zSubject->save();
         $zUpload->remove();
         $zPlugin->do( 'zeromin.remove.thread', $iBBS, [$key] );
@@ -186,7 +189,7 @@ sub repair {
 }
 
 sub repair_upload_file_table {
-    my ($zThread) = @_;
+    my ( $zThread, $plugin_iterator ) = @_;
     my $zUpload = $zThread->{__upload};
 
     if ( !$zUpload ) {
@@ -195,14 +198,29 @@ sub repair_upload_file_table {
             { id => $zThread->{_bbs} } );
         $zThread->{__upload} = $zUpload = Zeromin::Upload->new( $iBBS, 0 );
     }
-    $zUpload->set_key( $zThread->{_key} );
+
+    $plugin_iterator ||= $zThread->{__plugin};
+    if ( !$plugin_iterator ) {
+        require Zeromin::Plugin;
+        my $iBBS = Img0ch::BBS->new( $zThread->{_kernel},
+            { id => $zThread->{_bbs} } );
+        my $zPlugin = Zeromin::Plugin->new($iBBS);
+        $plugin_iterator
+            = $zPlugin->iterator( 'zeromin.repair.upload', sub { } );
+        $zThread->{__plugin} = $plugin_iterator;
+    }
+
+    my $key = $zThread->{_key};
+    $zUpload->set_key($key);
     $zUpload->repair();
+    $plugin_iterator->($key);
+    return 1;
 }
 
 sub import_from {
     my ( $zThread, $path, $detect_hint ) = @_;
     my $iKernel = $zThread->{_kernel};
-    my $iBBS    = Img0ch::BBS->new( $iKernel, { id => $zThread->{_bbs} } );
+    my $iBBS = Img0ch::BBS->new( $iKernel, { id => $zThread->{_bbs} } );
     $detect_hint ||= $path;
 
     require File::Copy;
@@ -214,18 +232,19 @@ sub import_from {
         _strip_tags_from_dat( $iKernel, $dest );
         return 1;
     }
-    elsif ( $detect_hint =~ /\.tar\.gz\z/xms
-        or $detect_hint =~ /\.tgz\z/xms ) {
+    elsif ($detect_hint =~ /\.tar\.gz\z/xms
+        or $detect_hint =~ /\.tgz\z/xms )
+    {
         require Archive::Tar;
         require File::Temp;
-        my $tar  = Archive::Tar->new($path, 1)
-            or $iKernel->throw_exception(Archive::Tar->error());
-        my $cwd  = File::Temp::tempdir( CLEANUP => 1 );
+        my $tar = Archive::Tar->new( $path, 1 )
+            or $iKernel->throw_exception( Archive::Tar->error() );
+        my $cwd = File::Temp::tempdir( CLEANUP => 1 );
         $tar->setcwd($cwd);
-        my @list = $tar->extract(
-            grep { $_->full_path() =~ /\d{9,10}\.dat\z/xms
-            } $tar->get_files());
-        for my $member ( @list ) {
+        my @list
+            = $tar->extract( grep { $_->full_path() =~ /\d{9,10}\.dat\z/xms }
+                $tar->get_files() );
+        for my $member (@list) {
             my $dat    = join '/', $cwd, $member->full_path();
             my $suffix = File::Basename::basename($dat);
             my $dest   = $iBBS->path( 'dat', $suffix );
@@ -236,20 +255,23 @@ sub import_from {
         return 1;
     }
     elsif ( eval "require Compress::Zlib; 1"
-        and $detect_hint =~ /\.zip/xms ) {
+        and $detect_hint =~ /\.zip/xms )
+    {
         require Archive::Zip;
-        Archive::Zip::setErrorHandler(sub {
-            my $error = shift;
-            $iKernel->throw_exception($error);
-        });
+        Archive::Zip::setErrorHandler(
+            sub {
+                my $error = shift;
+                $iKernel->throw_exception($error);
+            }
+        );
         my $zip     = Archive::Zip->new($path);
         my @members = $zip->membersMatching('\d{9,10}\.dat');
         for my $member (@members) {
             my ( undef, $temp ) = Archive::Zip::tempFile();
-            $member->extractToFileNamed($temp)
-                == Archive::Zip::AZ_OK() or $iKernel->throw_exception($!);
-            my $suffix = File::Basename::basename($member->fileName());
-            my $dest   = $iBBS->path( 'dat', $suffix );
+            $member->extractToFileNamed($temp) == Archive::Zip::AZ_OK()
+                or $iKernel->throw_exception($!);
+            my $suffix = File::Basename::basename( $member->fileName() );
+            my $dest = $iBBS->path( 'dat', $suffix );
             File::Copy::copy( $temp, $dest )
                 or $iKernel->throw_io_exception($temp);
             _strip_tags_from_dat( $iKernel, $dest );
@@ -282,9 +304,9 @@ sub _strip_tags_from_dat {
     my $temp = "${dest}.tmp";
     eval {
         local ( *RFH, *WFH, $! );
-        open *RFH, "<${dest}" ## no critic
+        open *RFH, "<${dest}"    ## no critic
             or $iKernel->throw_io_exception($dest);
-        open *WFH, ">${temp}" ## no critic
+        open *WFH, ">${temp}"    ## no critic
             or $iKernel->throw_io_exception($temp);
         while ( my $line = <RFH> ) {
             $line =~ s{</?a[^>]+>}{}gxms;
